/*<ATGCOPYRIGHT>
 * Copyright (C) 2004 Art Technology Group, Inc.
 * All Rights Reserved.  No use, copying or distribution of this
 * work may be made except in accordance with a valid license
 * agreement from Art Technology Group.  This notice must be
 * included on all copies, modifications and derivatives of this
 * work.
 *
 * Art Technology Group (ATG) MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. ATG SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
 * MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * "Dynamo" is a trademark of Art Technology Group, Inc.
 </ATGCOPYRIGHT>*/

package com.atg.ant.taskdefs;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.DirSet;


/**
 * GenerateSQLControlBlock is an ant task which will generate a control
 * script which simply invokes individual sql scripts in the directories
 * specified.
 *
 * This task is meant to be called multiple times to create the final
 * script.  The first invocation should create the file, while subsequent
 * ones will append to the control script (using the create attribute).
 *
 * It assumes that the following environment variables will be available
 *
 * <li> ENV_NAME -- used for the if stmt
 * <li> ORACLE_LOGS
 * <li> ORACLE_SID
 * <li> ORACLE_BIN_SQLPLUS -- sqlplus executable
 * <li> <schemaName> -- holds the user/pass for that schema
 *
 */
public class GenerateSQLControlBlock extends Task {

    private File mScript;
    /**
     * setter for the script attribute
     */
    public void setScript (File pScript) {
	mScript = pScript;
    }

    private Vector mDirSets = new Vector();
    /**
     * setter for the dirSets attribute
     */
    public void addDirset(DirSet pDirSet) {
	mDirSets.addElement(pDirSet);
    }

    private boolean mHaveCreateFlag = false;
    private boolean mCreateFlag;
    /**
     * setter for createFlag attribute
     */
    public void setCreate (boolean create) {
	mHaveCreateFlag = true;
	mCreateFlag = create;
    }

    private boolean isEnvSpecific = false;
    /**
     * setter for envSpecific attribute
     */
    public void setEnvSpecific (boolean envSpecific) {
	isEnvSpecific = envSpecific;
    }

    private String mEnv;
    /**
     * setter for the env attribute
     */
    public void setEnv (String pEnv) {
	mEnv = pEnv;
    }


    /**
     * execute the task
     *
     * @throws BuildException generally a wrapped IOException
     */
    public void execute ()
	throws BuildException
    {
	DirSet ds;
	String tmpFiles[];
	String indent = "";  // used to indent the script
	String schema;
	int idx;

	// check for create as a required attribute
	//
	if (!mHaveCreateFlag) {
	    throw new BuildException("Required attribute create not set",
				     getLocation());
	}

	try {
	    // append to the file
	    //
	    BufferedWriter script =
		new BufferedWriter(new FileWriter(mScript.getAbsolutePath(), !mCreateFlag));

	    // header comment
	    //
	    if (mCreateFlag) {
		script.write("# auto-generated sql control scriptlet");
		script.newLine();
		script.write("#");
		script.newLine();
		script.newLine();
	    }

	    // conditionally output an environmental "if" stmt
	    //
	    if (isEnvSpecific) {
		script.write("if [ $ENV_NAME = '" + mEnv + "' ] then ;");
		script.newLine();
		indent = "  ";
	    }

	    // for each directory
	    //
	    for (int i=0; i < mDirSets.size(); i++) {
		ds = (DirSet) mDirSets.elementAt(i);
		tmpFiles = ds.getDirectoryScanner(getProject()).getIncludedFiles();
		
		// for each file in that directory
		//
		for (int j=0; j < tmpFiles.length; j++) {

		    // get the schema name
		    // it's the variable name for the user/pass
		    //
		    idx = tmpFiles[j].indexOf(File.separatorChar);
		    schema = tmpFiles[j].substring(0, idx);

		    // output a command to invoke that script
		    //
		    script.write(indent +
				 "$ORACLE_BIN_SQLPLUS -s " +
				 "$" + schema + "@$ORACLE_SID \\");
		    script.newLine();
		    script.write(indent + "  @" + tmpFiles[j]);
		    script.write(" >> $LOGS 2>&1");
		    script.newLine();
		    script.newLine();
		}
	    }

	    if (isEnvSpecific) {
		script.write("fi");
		script.newLine();
		script.newLine();
	    }

	    script.close();
	}
	catch (IOException ioe) {
	    System.err.println(ioe);
	    throw new BuildException("Error writing to " + mScript);
	}
    }
}
