/*<ATGCOPYRIGHT>
 * Copyright (C) 2004 Art Technology Group, Inc.
 * All Rights Reserved.  No use, copying or distribution of this
 * work may be made except in accordance with a valid license
 * agreement from Art Technology Group.  This notice must be
 * included on all copies, modifications and derivatives of this
 * work.
 *
 * Art Technology Group (ATG) MAKES NO REPRESENTATIONS OR WARRANTIES
 * ABOUT THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. ATG SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
 * MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * "Dynamo" is a trademark of Art Technology Group, Inc.
 </ATGCOPYRIGHT>*/

package com.atg.ant.taskdefs;

import java.io.File;
import java.util.StringTokenizer;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Execute;
import org.apache.tools.ant.taskdefs.ExecuteStreamHandler;
import org.apache.tools.ant.taskdefs.LogStreamHandler;
import org.apache.tools.ant.types.Commandline;


/**
 * PcliRun is an ant task which will run pcli scripts
 */
public class PcliRun extends Task {

    private String pvcsbin;
    private String repository;
    private String pvcsProject;
    private String userId;
    private boolean ignorerc;

    /**
     * Constant for the thing to execute
     */
    private static final String PCLI_EXE = "pcli";

    /**
     * Constant for the thing to execute
     */
    private static final String GET_EXE = "get";


    protected int runCmd(Commandline cmd, ExecuteStreamHandler out) {
        try {
            Project aProj = getProject();
            Execute exe = new Execute(out);
            exe.setAntRun(aProj);
            exe.setWorkingDirectory(aProj.getBaseDir());
            exe.setCommandline(cmd.getCommandline());
            return exe.execute();
        } catch (java.io.IOException e) {
            String msg = "Failed executing: " + cmd.toString()
                + ". Exception: " + e.getMessage();
            throw new BuildException(msg, getLocation());
        }
    }

    private String getExecutable(String exe) {
        StringBuffer correctedExe = new StringBuffer();
        if (getPvcsbin() != null) {
            if (pvcsbin.endsWith(File.separator)) {
                correctedExe.append(pvcsbin);
            } else {
                correctedExe.append(pvcsbin).append(File.separator);
            }
        }
        return correctedExe.append(exe).toString();
    }

    /**
     * @exception org.apache.tools.ant.BuildException Something is stopping the build...
     */
    public void execute() throws org.apache.tools.ant.BuildException {
        int result = 0;

        if (getUserId() == null || getUserId().trim().equals("")) {
            throw new BuildException("Required argument userId not specified");
        }
        if (getRepository() == null || getRepository().trim().equals("")) {
            throw new BuildException("Required argument repository not specified");
        }
        if (getPvcsproject() == null || getPvcsproject().trim().equals("")) {
            throw new BuildException("Required argument pvcsproject not specified");
        }

        Commandline commandLine = new Commandline();
        commandLine.setExecutable(getExecutable(PCLI_EXE));

	commandLine.createArgument().setValue("run");

	commandLine.createArgument().setValue("-s" + getScriptName());

	commandLine.createArgument().setValue(getUserId());

	commandLine.createArgument().setValue(getRepository());

	commandLine.createArgument().setValue(getPvcsproject());

	StringTokenizer strtok = new StringTokenizer(getScriptArguments());
	while (strtok.hasMoreTokens())
	    commandLine.createArgument().setValue(strtok.nextToken());

	System.out.println(getScriptName());
	result = runCmd(commandLine,
			new LogStreamHandler(this,
					     Project.MSG_INFO,
					     Project.MSG_WARN));
	if (result != 0) {

	    if (ignorerc) {

		// if the user supplies a prop name, store the result
		//
		if (getResultPropName() != null &&
		    !getResultPropName().trim().equals("")) {

		    getProject().setProperty(getResultPropName(), result + "");
		}
		// else drop the result

	    } else {
		String msg = "Failed executing: " + commandLine.toString()
		    + ". Return code was " + result;
		throw new BuildException(msg, getLocation());
	    }
	}

    }

    /**
     * Get network name of the PVCS repository
     * @return String
     */
    public String getRepository() {
        return repository;
    }
    /**
     * The network name of the PVCS repository; required.
     * @param repo String
     */
    public void setRepository(String repo) {
        repository = repo;
    }

    /**
     * Get name of the project in the PVCS repository
     * @return String
     */
    public String getPvcsproject() {
        return pvcsProject;
    }

    /**
     * The project within the PVCS repository to extract files from;
     * optional, default &quot;/&quot;
     * @param prj String
     */
    public void setPvcsproject(String prj) {
        pvcsProject = prj;
    }


    /**
     * Get name of the PVCS bin directory
     * @return String
     */
    public String getPvcsbin() {
        return pvcsbin;
    }

    /**
     * Specifies the location of the PVCS bin directory; optional if on the PATH.
     * On some systems the PVCS executables <i>pcli</i>
     * and <i>get</i> are not found in the PATH. In such cases this attribute
     * should be set to the bin directory of the PVCS installation containing
     * the executables mentioned before. If this attribute isn't specified the
     * tag expects the executables to be found using the PATH environment variable.
     * @param bin PVCS bin directory
     * @todo use a File setter and resolve paths.
     */
    public void setPvcsbin(String bin) {
        pvcsbin = bin;
    }


    /**
     * Get value of ignorereturncode
     * @return String
     */
    public boolean getIgnoreReturnCode() {
        return ignorerc;
    }

    /**
     * If set to true the return value from executing the pvcs
     * commands are ignored; optional, default false.
     */
    public void setIgnoreReturnCode(boolean b) {
        ignorerc = b;
    }

    public String getUserId() {
        return userId;
    }

    /**
     * User ID; unused.
     * @ant.attribute ignore="true"
     */

    public void setUserId(String u) {
        userId = u;
    }


    private String mScriptName;
    public void setScriptName(String pScriptName) {
	mScriptName = pScriptName;
    }
    public String getScriptName() {
	return mScriptName;
    }

    private String mScriptArguments;
    public void setScriptArguments(String pScriptArguments) {
	mScriptArguments = pScriptArguments;
    }
    public String getScriptArguments() {
	return mScriptArguments;
    }


    private String mResultPropName;
    public void setResultPropName(String pResultPropName) {
	mResultPropName = pResultPropName;
    }
    public String getResultPropName() {
	return mResultPropName;
    }



    /**
     * Creates a PcliRun object
     */
    public PcliRun() {
        super();
        pvcsProject = null;
        repository = null;
        pvcsbin = null;
        ignorerc = false;
    }
}

